/**
 * Creates a grid of forms
 */



class Anime {

  float xGrid, yGrid;
  float posX, posY;
  int surfaceW, surfaceH;
  int marginTop, marginBottom, marginLeft, marginRight;
  float theta;
  float phi;
  float thetaOffSet = 1.0;
  float freqX, ampX, freqY, ampY;

  Anime() {
  }

  /**
   * Sets the parameters for the animation surface
   *
   * @param surfaceW : width
   * @param surfaceH : height
   * @param mTop : top & bottom margins
   * @param mSides : side margins
   *
   */

  void setGridParameters(int w, int h, int mTop, int mSides) {
    this.surfaceW = w;
    this.surfaceH = h;
    this.marginTop = marginBottom = mTop;
    this.marginLeft = marginRight = mSides;
    this.theta = 0.0;
  }

  /**
   * Draws the animation
   * @param x : x coordinate
   * @param y : y coordinate
   */

  void drawAnime(float surfaceX, float surfaceY) {
    pushMatrix();
    translate(surfaceX, surfaceY);
    for (int j=marginTop; j<surfaceH-marginBottom; j+=yInterval) {
      float dphi = (TWO_PI/ (surfaceW-(marginTop+marginBottom)))*yInterval; 
      for (int i=marginLeft; i<surfaceW-marginRight; i+=xInterval) {
        xGrid = i+cos(this.phi*this.freqX)*this.ampX;
        //xGrid = i;
        yGrid = j+sin(this.phi*this.freqY)*this.ampY;

        float dist = dist(xGrid, yGrid, surfaceW/2, surfaceH/2);
        float hue = map(dist, 0, surfaceH*2, colorHueMin, colorHueMax);

        if (isAlpha) {
          
          float alphaRange = colorHueMax - colorHueMin;
          float alpha = map(sin((phi*waveFreqX)+(xGrid*alphaRange)), -1, 1, 0, 100);
          if (isColour) {
            stroke(hue, 100, 100, alpha);
          } else {
            stroke(hue, 0, 100, alpha);
          }
        } else {
          if (isColour) {
            stroke(hue, 100, 100);
          } else {
            stroke(hue, 0, 100);
          }
        }
        strokeWeight( strokeW );
        pushMatrix();
        translate(xGrid, yGrid);

        if (isKenPerlin) {
          float snx = sin(frameCount * 0.0007) * 6; // Smoother
          float sny = sin(frameCount * 0.0003) * 6; // Smoother

          float pr = noise(xGrid * noiseScale + snx, yGrid * noiseScale + sny) * noiseAmm;
          rotate(radians(pr));
        } else {       
          rotate(radians(rotateAngle));
        }

        line(-lineLen, -lineLen, lineLen, lineLen);
        popMatrix();
        phi += dphi;
      }
    }
    popMatrix();
  }

  /**
   * Updates freq & amp values
   * @param f : frequency
   * @param a : amplitude
   */

  void applySinMotion(float f, float a) {
    this.freqX = f;
    this.ampX = a;
  }
  
    /**
   * Updates freq & amp values
   * @param f : frequency
   * @param a : amplitude
   */

  void applyCosMotion(float f, float a) {
    this.freqY = f;
    this.ampY = a;
  }


  // ... other methods...
  /**
   * 
   */
  void applyNoise() {
  }


  void update() {
    this.theta += thetaOffSet;
    this.phi = theta;
  }
}