/**
 *
 * GUI : Program Interface
 */
 
 import controlP5.*;
 PrintWriter output; 

 ///////////////// GLOBAL VARIABLES TO GUI :
 ControlP5 cp5;
 float waveAmpX, waveFreqX;
 float waveAmpY, waveFreqY;
 float lineLen, strokeW, rotateAngle;
 int xInterval, yInterval;
 float noiseScale, noiseAmm;
 
 int colorHueMin = 0;
 int colorHueMax = 360;
 
 boolean isAlpha, isColour;
 boolean isKenPerlin = false;
 boolean isGUI = true;
 
 
 void initGUI() {
    cp5 = new ControlP5(this);

  ///////////////////////////////////////////////////// GEOMETRY

  Group g1  =  cp5.addGroup("g1")
    .setPosition(25, 30).setWidth(250).setBackgroundHeight(320)
    .activateEvent(true).setLabel("FORM")
    .setBackgroundColor(color( #CBCBCB, 100 ))
    .setBarHeight(20);
              
  cp5.addSlider("lineLen")
      .setPosition(10, 10)
        .setSize(140, 25)
          .setRange(0.0f, 200.0f)
            .setValue(30.0f).setGroup(g1);            
  
   cp5.addSlider("strokeW")
      .setPosition(10, 45)
        .setSize(140, 25)
          .setRange(0.0f, 200.0f)
            .setValue(12.0f).setGroup(g1);            
              
   cp5.addSlider("xInterval")
      .setPosition(10, 80)
        .setSize(140, 25)
          .setRange(3, 100)
            .setValue(40).setGroup(g1); 
            
    cp5.addSlider("yInterval")
      .setPosition(10, 115)
        .setSize(140, 25)
          .setRange(3, 100)
            .setValue(30).setGroup(g1); 
            
           
   Numberbox nb1 = cp5.addNumberbox("rotateAngle").setPosition(10, 150)
      .setRange(0.0, 360.0).setValue(45.0f).setGroup(g1)
        .setColorBackground(#F5F5F5).setColorForeground(#A09E9F)
          .setColorValueLabel(#525252).setColorActive(#F08D0C);
              makeEditable( nb1 );
  
  
  cp5.addToggle("isAlpha")
    .setPosition(50,225)
      .setSize(20,20)
        .setValue(false)
          .setGroup(g1);
          
 cp5.addToggle("isColour")
    .setPosition(10,225)
      .setSize(20,20)
        .setValue(false)
          .setGroup(g1);
          
          
  cp5.addRange("rangeController")
             // disable broadcasting since setRange and setRangeValues will trigger an event
             .setBroadcast(false) 
             .setPosition(10,270)
             .setSize(160,25)
             .setHandleSize(20)
             .setRange(0,360)
             .setRangeValues(280,320)
             // after the initialization we turn broadcast back on again
             .setBroadcast(true)
             .setColorForeground(color(0,255,0,140))
             .setColorBackground(color(0,133))
             .setGroup(g1);
             ;
                     
        
 
   ///////////////////////////////////////////////////// NOISE

  Group g2  =  cp5.addGroup("g2")
    .setPosition(310, 30).setWidth(400).setBackgroundHeight(220)
    .activateEvent(true).setLabel("MOTION")
    .setBackgroundColor(color( #CBCBCB, 100 ))
    .setBarHeight(20);
    
  cp5.addSlider("waveAmpX")
      .setPosition(10, 60)
        .setSize(140, 25)
          .setRange(0.0f, 300.0f)
            .setValue(50.0f).setGroup(g2);

   cp5.addSlider("waveFreqX")
      .setPosition(10, 100)
        .setSize(140, 25)
          .setRange(0.00f, 0.09f)
            .setValue(0.065f).setGroup(g2);   

  cp5.addSlider("waveAmpY")
      .setPosition(10, 140)
        .setSize(140, 25)
          .setRange(0.0f, 300.0f)
            .setValue(0.0f).setGroup(g2);

   cp5.addSlider("waveFreqY")
      .setPosition(10, 180)
        .setSize(140, 25)
          .setRange(0.00f, 0.09f)
            .setValue(0.01f).setGroup(g2);   

cp5.addToggle("isKenPerlin")
     .setPosition(200,20)
      .setSize(20,20)
        .setValue(false).setGroup(g2); 
       
  cp5.addSlider("noiseScale")
    .setPosition(200,60)
       .setSize(140, 25)
          .setRange(0.000f, 0.05f)
            .setValue(0.005f).setGroup(g2); 
 
 cp5.addSlider("noiseAmm")
    .setPosition(200,100)
       .setSize(140, 25)
          .setRange(0, 300)
            .setValue(15).setGroup(g2);   
 
            
    
  
  cp5.addBang("bangSave").setPosition(10, 20).setGroup(g2)   
    .setSize(20, 20).setTriggerEvent(Bang.RELEASE).setCaptionLabel("save")
    .setColorForeground(color(255, 255, 0, 150))
    .setColorActive(color(255, 255, 0))
    ; 
    
  cp5.addBang("bangLoad").setPosition(50, 20).setGroup(g2)
    .setSize(20, 20).setTriggerEvent(Bang.RELEASE).setCaptionLabel("load")
    .setColorForeground(color(255, 255, 0, 150))
    .setColorActive(color(255, 255, 0)); 
                  
           
          
  }
  
  
  /// GUI specific functions
  
void controlEvent(ControlEvent theControlEvent) {
  if(theControlEvent.isFrom("rangeController")) {
    // min and max values are stored in an array.
    // access this array with controller().arrayValue().
    // min is at index 0, max is at index 1.
    colorHueMin = int(theControlEvent.getController().getArrayValue(0));
    colorHueMax = int(theControlEvent.getController().getArrayValue(1));
    //println("range update, done.");
  }
  
}  


////////// numberbox handler
// function that will be called when controller 'numbers' changes
public void numbers(float f) {
  println("received "+f+" from Numberbox numbers ");
}

void makeEditable( Numberbox n ) {
  // allows the user to click a numberbox and type in a number which is confirmed with RETURN
  final NumberboxInput nin = new NumberboxInput( n ); // custom input handler for the numberbox

  // control the active-status of the input handler when releasing the mouse button inside
  // the numberbox. deactivate input handler when mouse leaves.
  n.onClick(new CallbackListener() {
    public void controlEvent(CallbackEvent theEvent) {
      nin.setActive( true );
    }
  }
  ).onLeave(new CallbackListener() {
    public void controlEvent(CallbackEvent theEvent) {
      nin.setActive( false );
      nin.submit();
    }
  }
  );
}


// input handler for a Numberbox that allows the user to
// key in numbers with the keyboard to change the value of the numberbox

public class NumberboxInput {
  String text = "";
  Numberbox n;
  boolean active;

  NumberboxInput(Numberbox theNumberbox) {
    n = theNumberbox;
    registerMethod("keyEvent", this );
  }

  public void keyEvent(KeyEvent k) {
    // only process key event if input is active
    if (k.getAction()==KeyEvent.PRESS && active) {
      if (k.getKey()=='\n') { // confirm input with enter
        submit();
        return;
      } else if (k.getKeyCode()==BACKSPACE) {
        text = text.isEmpty() ? "":text.substring(0, text.length()-1);
        //text = ""; // clear all text with backspace
      } else if (k.getKey()<255) {
        // check if the input is a valid (decimal) number
        final String regex = "\\d+([.]\\d{0,2})?";
        String s = text + k.getKey();
        if ( java.util.regex.Pattern.matches(regex, s ) ) {
          text += k.getKey();
        }
      }
      n.getValueLabel().setText(this.text);
    }
  }

  public void setActive(boolean b) {
    active = b;
    if (active) {
      n.getValueLabel().setText("");
      text = "";
    }
  }

  public void submit() {
    if (!text.isEmpty()) {
      n.setValue( float( text ) );
      text = "";
    } else {
      n.getValueLabel().setText(""+n.getValue());
    }
  }
}